@@ -702,14 +702,19 @@ fn replace_wildcard_with_column(expr: Expr, column_name: Arc<String>) -> Expr {
             truthy: Box::new(replace_wildcard_with_column(*truthy, column_name.clone())),
             falsy: Box::new(replace_wildcard_with_column(*falsy, column_name)),
         },
-        Expr::Udf {
+        Expr::Function {
             input,
             function,
             output_type,
-        } => Expr::Udf {
-            input: Box::new(replace_wildcard_with_column(*input, column_name)),
+            collect_groups,
+        } => Expr::Function {
+            input: input
+                .into_iter()
+                .map(|e| replace_wildcard_with_column(e, column_name.clone()))
+                .collect(),
             function,
             output_type,
+            collect_groups,
         },
         Expr::BinaryFunction {
             input_a,
@@ -1357,10 +1362,10 @@ mod test {
 
         // check if optimizations succeeds without selection
         {
-            let lf =
-                left.clone()
-                    .lazy()
-                    .left_join(right.clone().lazy(), col("days"), col("days"), None);
+            let lf = left
+                .clone()
+                .lazy()
+                .left_join(right.clone().lazy(), col("days"), col("days"));
 
             print_plans(&lf);
             // implicitly checks logical plan == optimized logical plan
@@ -1373,7 +1378,7 @@ mod test {
             let lf = left
                 .clone()
                 .lazy()
-                .left_join(right.clone().lazy(), col("days"), col("days"), None)
+                .left_join(right.clone().lazy(), col("days"), col("days"))
                 .select(&[col("temp")]);
 
             print_plans(&lf);
@@ -1386,7 +1391,7 @@ mod test {
             let lf = left
                 .clone()
                 .lazy()
-                .left_join(right.clone().lazy(), col("days"), col("days"), None)
+                .left_join(right.clone().lazy(), col("days"), col("days"))
                 .select(&[col("temp"), col("rain_right")]);
 
             print_plans(&lf);
